package ru.dbotthepony.kstarbound.client.gl.vertex

import org.lwjgl.opengl.GL46
import ru.dbotthepony.kstarbound.client.gl.GLStateTracker
import ru.dbotthepony.kstarbound.client.gl.checkForGLError
import java.io.Closeable

class StatefulVertexBuilder(
	val state: GLStateTracker,
	attributes: GLAttributeList,
	type: GeometryType,
) : HeapVertexBuilder<StatefulVertexBuilder>(attributes, type), Closeable {
	private val vao = state.newVAO()
	private val vbo = state.newVBO()
	private val ebo = state.newEBO()

	init {
		vao.bind()
		vbo.bind()
		ebo.bind()

		attributes.apply(vao, true)

		vao.unbind()
		vbo.unbind()
		ebo.unbind()
	}

	fun upload(drawType: Int = GL46.GL_STATIC_DRAW) {
		upload(vbo, ebo, drawType)
	}

	fun bind() = vao.bind()
	fun unbind() = vao.unbind()

	fun draw(primitives: Int = GL46.GL_TRIANGLES) {
		bind()
		GL46.glDrawElements(primitives, indexCount, elementIndexType, 0L)
		checkForGLError()
	}

	override fun close() {
		vao.close()
		vbo.close()
		ebo.close()
	}
}
