package ru.dbotthepony.kstarbound.defs

import com.google.common.collect.ImmutableList
import com.google.common.collect.ImmutableMap

/**
 * Возвращает глубокую неизменяемую копию [input] примитивов/List'ов/Map'ов
 */
fun enrollList(input: List<Any>, interner: (String) -> String = String::intern): ImmutableList<Any> {
	val builder = ImmutableList.builder<Any>()

	for (v in input) {
		when (v) {
			is Map<*, *> -> builder.add(enrollMap(v as Map<String, Any>, interner))
			is List<*> -> builder.add(enrollList(v as List<Any>, interner))
			else -> builder.add((v as? String)?.let(interner) ?: v)
		}
	}

	return builder.build()
}

/**
 * Возвращает глубокую неизменяемую копию [input] примитивов/List'ов/Map'ов
 */
fun enrollMap(input: Map<String, Any>, interner: (String) -> String = String::intern): ImmutableMap<String, Any> {
	val builder = ImmutableMap.builder<String, Any>()

	for ((k, v) in input) {
		when (v) {
			is Map<*, *> -> builder.put(interner(k), enrollMap(v as Map<String, Any>, interner))
			is List<*> -> builder.put(interner(k), enrollList(v as List<Any>, interner))
			else -> builder.put(interner(k), (v as? String)?.let(interner) ?: v)
		}
	}

	return builder.build()
}
