package ru.dbotthepony.kstarbound.defs.world.dungeon

import com.google.gson.GsonBuilder
import ru.dbotthepony.kstarbound.defs.world.SkyParameters
import ru.dbotthepony.kstarbound.defs.world.WorldProperties
import ru.dbotthepony.kstarbound.io.json.CustomEnumTypeAdapter
import ru.dbotthepony.kstarbound.io.json.BuilderAdapter
import kotlin.properties.Delegates

class DungeonWorldDef {
	var type by Delegates.notNull<DungeonType>()
	var dungeonWorld by Delegates.notNull<String>()
	var seed by Delegates.notNull<Long>()

	var spawningEnabled = false
	var beamUpRule by Delegates.notNull<BeamUpRule>()

	var disableDeathDrops = true

	var worldProperties: WorldProperties? = null
	var skyParameters = SkyParameters()

	/**
	 * Milliseconds
	 */
	var tempWorldDeleteTime = -1L

	var persistent = false
	var useUniverseClock = false

	var planetType: String? = null
	var planetSize: String? = null

	companion object {
		val ADAPTER = BuilderAdapter(::DungeonWorldDef,
			DungeonWorldDef::type,
			DungeonWorldDef::dungeonWorld,
			DungeonWorldDef::seed,
			DungeonWorldDef::spawningEnabled,
			DungeonWorldDef::beamUpRule,
			DungeonWorldDef::disableDeathDrops,
			DungeonWorldDef::worldProperties,
			DungeonWorldDef::skyParameters,
			DungeonWorldDef::tempWorldDeleteTime,
			DungeonWorldDef::persistent,
			DungeonWorldDef::useUniverseClock,
			DungeonWorldDef::planetType,
			DungeonWorldDef::planetSize,
		)

		fun registerGson(gsonBuilder: GsonBuilder) {
			gsonBuilder.registerTypeAdapter(DungeonWorldDef::class.java, ADAPTER)
			gsonBuilder.registerTypeAdapter(BeamUpRule::class.java, CustomEnumTypeAdapter(BeamUpRule.values()).nullSafe())
			gsonBuilder.registerTypeAdapter(DungeonType::class.java, CustomEnumTypeAdapter(DungeonType.values()).nullSafe())
		}
	}
}
